module Tagline
  class TagStorer
    def initialize(filename, lines)
      @filename, @lines = filename, lines
    end

    def retrieve_stored_tags
      tag_names.each do |tag|
        next unless File.exist?(tag_filename(tag))
        tagged_lines = File.read(tag_filename(tag)).split("\n")
        lines.each { |line| line[tag] = tagged_lines.include?(line.text) }
      end
    end

    def save_tags
      tag_names.each do |tag|
        tagged_lines = lines.select { |line| line[tag] }.map(&:text)
        next if tagged_lines.empty?
        File.write(tag_filename(tag), tagged_lines.join("\n"))
      end
    end

    private

    attr_reader :filename, :lines

    def tag_names
      @tag_names ||= Tagline.tags.keys
    end

    def file_ext
      @file_ext ||= File.extname(filename)
    end

    def file_without_ext
      @file_without_ext ||= filename.chomp(file_ext)
    end

    def tag_filename(tag)
      "#{file_without_ext}_#{tag}#{file_ext}"
    end
  end
end
