require 'tagline/character_reader'
require 'tagline/line_manager'
require 'tagline/screen_printer'
require 'tagline/tag_storer'

module Tagline
  class Tagger < CharacterReader
    extend Forwardable

    def initialize(filename)
      @filename = filename
      @line_manager = LineManager.new(filename)
      @tag_storer = TagStorer.new(filename, lines)
      retrieve_stored_tags
    end

    define_method(:before_loop) { print_screen }
    define_method(:up_key) { previous_line; print_screen }
    define_method(:down_key) { next_line; print_screen }
    define_method(:n_key) { Tagline.next_on_tag = !Tagline.next_on_tag }
    define_method(:after_loop) { save_tags }

    def tag(chord)
      tag = Tagline.tags.invert[chord]
      return unless tag
      toggle(tag)
      next_line if Tagline.next_on_tag
      print_screen
    end

    private

    attr_reader :filename, :line_manager, :tag_storer
    def_delegators :line_manager, :lines, :line, :next_line, :previous_line
    def_delegator :line, :toggle
    def_delegators :tag_storer, :retrieve_stored_tags, :save_tags
    define_method(:print_screen) { ScreenPrinter.print_screen(line) }
  end
end
